@@ -97,7 +97,7 @@ __CUSTOMER_PREPAID_DATA_TEMPLATE_UPSTREAM_SCHEMA = {
 __CUSTOMER_REDEMPTION_DATA_TEMPLATE_UPSTREAM_SCHEMA = {
                                             "CustomerKey"           : 'customer_key',
                                             "MerchantKey"           : 'merchant_key',
-                                            "RedeemedOutlet"          : 'redeemed_outlet_key',
+                                            "RedeemedOutlet"        : 'redeemed_outlet_key',
                                             "TransactionId"         : 'transaction_id',
                                             "RedeemedAmount"        : 'redeemed_amount',
                                             "RewardFormat"          : 'reward_format',
@@ -125,6 +125,7 @@ def __create_upstream(upstream_entity, upstream_template, dataset_name, table_na
     if upstream_entity:
         schema = upstream_schema_config.get(upstream_template)
         for upstrem_field_name, attr_name in schema.items():
+            logger.debug('upstream_entity classname=%s', upstream_entity.__class__.__name__)
             upstream_json[upstrem_field_name] = default_serializable(getattr(upstream_entity, attr_name))
     
     if streamed_datetime is None:
@@ -247,7 +248,7 @@ def create_merchant_customer_prepaid_reverted_upstream_for_merchant(transaction_
     __create_upstream(transaction_details_with_prepaid_details, MERCHANT_CUSTOMER_PREPAID_TEMPLATE, MERCHANT_DATASET, final_table_name, 
                       streamed_datetime=reverted_datetime, Reverted=True, RevertedDateTime=reverted_datetime) 
 
-def create_merchant_customer_redemption_upstream_for_merchant(customer_redemption, streamed_datetime=None):
+def create_merchant_customer_redemption_upstream_for_merchant(customer_redemption, streamed_datetime=None, reverted=False, reverted_datetime=None):
     if streamed_datetime is None:
         streamed_datetime = datetime.utcnow()
     
@@ -260,52 +261,18 @@ def create_merchant_customer_redemption_upstream_for_merchant(customer_redemptio
     
     upstream_data_list = []
     
-    transaction_id      = customer_redemption.transaction_id
-    redeemed_outlet_key = customer_redemption.redeemed_outlet_key
-    merchant_key        = customer_redemption.redeemed_merchant_acct_key
-    customer_key        = customer_redemption.redeemed_customer_key
-    redeemed_amount     = customer_redemption.redeemed_amount
-    reward_format       = customer_redemption.reward_format
-    redeemed_datetime   = customer_redemption.redeemed_datetime
+    if customer_redemption.reward_format in (program_conf.REWARD_FORMAT_POINT, program_conf.REWARD_FORMAT_STAMP, program_conf.REWARD_FORMAT_PREPAID):
+        upstream_data_list.append(customer_redemption.to_upstream_info())
     
-    for reward_format, redemption_details in customer_redemption.redeemed_summary.items():
-        if reward_format in (program_conf.REWARD_FORMAT_POINT, program_conf.REWARD_FORMAT_STAMP, program_conf.REWARD_FORMAT_PREPAID):
-            upstream_data_list.append(CustomerRedeemedItemUpstream(
-                                        customer_key        = customer_key,
-                                        merchant_key        = merchant_key,
-                                        redeemed_outlet_key = redeemed_outlet_key,
-                                        transaction_id      = transaction_id,
-                                        redeemed_amount     = redeemed_amount,
-                                        reward_format       = reward_format,
-                                        redeemed_datetime   = redeemed_datetime,
-                                        
-                                        
-                                    ))
-        else:
-            for voucher_key, voucher_count in redemption_details.get('vouchers').items():
-                upstream_data_list.append(CustomerRedeemedItemUpstream(
-                                        customer_key        = customer_key,
-                                        merchant_key        = merchant_key,
-                                        redeemed_outlet_key = redeemed_outlet_key,
-                                        transaction_id      = transaction_id,
-                                        redeemed_amount     = voucher_count,
-                                        reward_format       = program_conf.REWARD_FORMAT_VOUCHER,
-                                        voucher_key         = voucher_key,
-                                        redeemed_datetime   = redeemed_datetime,
-                                        
-                                        
-                                    ))
+    elif customer_redemption.reward_format == program_conf.REWARD_FORMAT_VOUCHER:
+        upstream_data_list.extend(customer_redemption.to_voucher_upstream_info_list())
+            
     for upstream_data in upstream_data_list:
         __create_upstream(upstream_data, MERCHANT_CUSTOMER_REDEMPTION_TEMPLATE, MERCHANT_DATASET, final_table_name, 
-                      streamed_datetime=streamed_datetime, Reverted=False, RevertedDateTime=None)
+                      streamed_datetime=streamed_datetime, Reverted=reverted, RevertedDateTime=reverted_datetime)
     
 def create_merchant_customer_redemption_reverted_upstream_for_merchant(redemption_details, reverted_datetime):
-    partition_datetime = redemption_details.redeemed_datetime
-    
-    table_name          = MERCHANT_CUSTOMER_REDEMPTION_TEMPLATE
-    merchant_acct       = redemption_details.redeemed_merchant_acct
-    account_code        = merchant_acct.account_code.replace('-','')
-    final_table_name    = '{}_{}_{}'.format(table_name, account_code, partition_datetime.strftime('%Y%m%d'))
-    
-    __create_upstream(redemption_details, MERCHANT_CUSTOMER_REDEMPTION_TEMPLATE, MERCHANT_DATASET, final_table_name, 
-                      streamed_datetime=reverted_datetime, Reverted=True, RevertedDateTime=reverted_datetime)    
+    create_merchant_customer_redemption_upstream_for_merchant(redemption_details, 
+                                                              streamed_datetime=redemption_details.redeemed_datetime, 
+                                                              reverted=True, 
+                                                              reverted_datetime=redemption_details.reverted_datetime)     
