@@ -124,6 +124,20 @@ class QueryMerchantCustomerGrowthByYearMonth(QueryMerchantCustomDataByDateRangeB
             
         return query 
     
+    '''
+    def process_query_result(self, job_result_rows):
+        row_list = []
+        for row in job_result_rows:
+            #logger.debug(row)
+            column_dict = {}
+            column_dict[row.year_month]        = row.count
+            
+            row_list.append(column_dict)
+        
+        return row_list 
+    '''
+    def process_query_result(self, job_result_rows):
+        return process_job_result_into_category_and_count(job_result_rows)
     
 class QueryMerchantCustomerCountByDateRange(QueryMerchantCustomDataByDateRangeBaseResource):
     
