@@ -4,7 +4,8 @@ Created on 26 Jan 2021
 @author: jacklok
 '''
 
-from flask import Blueprint, render_template, session, abort, redirect, url_for, request, jsonify
+from flask import Blueprint, render_template, session, abort, redirect, url_for, request, jsonify, Response
+from flask_restful import Resource
 import logging, json, uuid
 from flask.globals import current_app
 from trexlib.utils.google.bigquery_util import create_bigquery_client, stream_data_by_datetime_partition, stream_data
@@ -29,6 +30,46 @@ logger = logging.getLogger('debug')
 
 import_upstream_data_api = Api(import_upstream_data_bp)
 
+class CheckUnsendUpstreamDataResource(Resource):
+    
+    def output_html(self, content, code=200, headers=None):
+        resp = Response(content, mimetype='text/html', headers=headers)
+        resp.status_code = code
+        return resp
+    
+    def post(self):
+        return self.get()
+    
+    def get(self):
+        try:
+            count = count_all_unsend_upstream_data()
+        except:
+            logger.debug(get_tracelog())
+        
+        return self.output_html("count=%d"% (count))
+    
+class ImportUpstreamDataResource(Resource):
+    
+    def output_html(self, content, code=200, headers=None):
+        resp = Response(content, mimetype='text/html', headers=headers)
+        resp.status_code = code
+        return resp
+    
+    def post(self):
+        return self.get()
+    
+    def get(self):
+        content         = request.args.to_dict() or request.json or {}
+        import_limit    = content.get('limit') or 1
+        imported_count  = 0
+        error_count     = 0
+        try:
+            (imported_count, error_count) = import_upstream_data_by_limit_function(limit = int(import_limit))
+        except:
+            logger.debug(get_tracelog())
+        
+        return self.output_html("imported_count=%d, error_count=%d, import datetim=%s"% (imported_count, error_count, datetime.now()))    
+
 class TriggerImportAllUpstreamData(TriggerImportDataBaseResource):
     
     def get_task_url(self):
@@ -40,7 +81,9 @@ class TriggerImportAllUpstreamData(TriggerImportDataBaseResource):
     
 class InitImportAllUpstreamData(InitImportDataBaseResource):
     def get_import_data_count(self, **kwargs):
-        return count_all_unsend_upstream_data()
+        count =  count_all_unsend_upstream_data()
+        logger.debug('InitImportAllUpstreamData debug: unsend upstream data count = %d', count)
+        return count
     
     def get_import_data_page_size(self):
         return analytics_conf.STREAM_DATA_PAGE_SIZE
@@ -209,6 +252,78 @@ def import_all_unsend_upstream_data_function(credential_info=None, dataset_name=
     return next_cursor
 
 
+def import_upstream_data_by_limit_function(credential_info=None, dataset_name='system_dataset', 
+                                            limit=analytics_conf.STREAM_DATA_PAGE_SIZE
+                                            ):
+    
+    
+    logger.debug('credential_filepath=%s', analytics_conf.BIGQUERY_SERVICE_CREDENTIAL_PATH)
+    bg_client       = create_bigquery_client(credential_filepath=analytics_conf.BIGQUERY_SERVICE_CREDENTIAL_PATH, project_id=analytics_conf.BIGQUERY_GCLOUD_PROJECT_ID)
+    
+    logger.debug('bg_client.project=%s', bg_client.project)
+    
+    if credential_info is None:
+        credential_info = current_app.config['credential_config']
+        
+    db_client = create_db_client(info=credential_info, caller_info="import_upstream_data_by_date_range_function")
+    import_upstream_dict = {}
+    
+    logger.debug('import_upstream_data_by_limit_function limit=%s', limit)
+    
+    
+    
+    with db_client.context():
+        upstream_data_list= UpstreamData.list_not_send(limit = limit)
+    
+    import_count    = len(upstream_data_list)
+    error_count     = 0
+    errors          = []    
+    imported_count  = 0
+    
+    for u in upstream_data_list:
+        
+        table_template_name = u.table_template_name
+        dataset_name        = u.dataset_name
+        table_name          = u.table_name
+        stream_content      = u.stream_content
+        
+        import_upstream_dict = {'': stream_content}
+
+
+    
+        logger.debug('################################ import_upstream_data_by_limit_function ############################################')
+        logger.debug('upstream data key=%s', u.key_in_str)
+        logger.debug(import_upstream_dict)
+        logger.debug('#####################################################################################################')
+        
+        
+        
+        __errors = stream_data(bg_client, dataset_name, table_template_name, 
+                                                   TABLE_SCHEME_TEMPLATE.get(table_template_name), table_name, stream_content
+                                                   )
+        
+        if len(__errors)>0:
+            errors.extend(__errors)
+            with db_client.context():
+                u.delete()
+            error_count+=1
+        else:
+            with db_client.context():
+                u.is_sent = True
+                u.put()
+                logger.debug("New rows have been added")
+                
+            imported_count+=1
+            
+    bg_client.close()
+    
+    if errors==[]:
+        logger.debug('imported_count=%d', imported_count)
+    else:
+        logger.debug("Encountered errors while inserting rows: {}".format(errors))
+    
+    return (imported_count, error_count)
+
 def import_upstream_data_by_date_range_function(date_start, date_end, credential_info=None, dataset_name='system_dataset', 
                                             limit=analytics_conf.STREAM_DATA_PAGE_SIZE, start_cursor=None, batch_id=None,
                                             
@@ -319,6 +434,9 @@ def read_upstream_data_content_by_date_range(date_start, date_end, limit=5):
     
     return 'Not found', 200
 
+
+import_upstream_data_api.add_resource(CheckUnsendUpstreamDataResource,         '/check-unsend-upstream-data')
+import_upstream_data_api.add_resource(ImportUpstreamDataResource,         '/import-unsend-upstream-data')
 import_upstream_data_api.add_resource(TriggerImportAllUpstreamData,         '/trigger-import-all-upstream-data')
 import_upstream_data_api.add_resource(InitImportAllUpstreamData,            '/init-import-all-upstream-data')
 import_upstream_data_api.add_resource(ImportAllUpstreamData,                '/import-all-upstream-data')
